@@ -77,10 +77,11 @@ indices needle@(Chunk n ns) _haystack@(Chunk k ks)
         go !(g::Int64) !i !msk !skp
             | i >= xlast = case xs of
                              Empty      -> (msk .|. swizzle z) :*: skp
-                             Chunk y ys -> buildTable y ys g 0 msk skp
-            | otherwise = go (g+1) (i+1) (msk .|. swizzle c) skp'
+                             Chunk y ys -> buildTable y ys g 0 msk' skp'
+            | otherwise = go (g+1) (i+1) msk' skp'
             where c                = A.unsafeIndex xarr (xoff+i)
-                  skp' | c == z    = nlen - fromIntegral g - 2
+                  msk'             = msk .|. swizzle c
+                  skp' | c == z    = nlen - g - 2
                        | otherwise = skp
                   xlast = xlen - 1
     -- | Check whether an attempt to index into the haystack at the
